/*  
 *  File:       SxNano_Grundstellung_V33.ino
 *  Autor:      Frank Keil
 *  Controller: Arduino Nano (5V, 16MHz), ATmega328
 *  Hardware:   Sx-Decoder von Frank Keil 
 *              http://frank-keil.de/Selectrix/Sx-Decoder/Sx-Decoder.html
 *  
 *  Libraries:  SXArduino: Quellen von G.v.d.Sel  
 *              https://github.com/gvandersel/SXArduino
 *
 *  Created on: 02.01.2012
 *  Last changed on: 13.06.2021
 *  
 *  Beschreibung:
 *  Schreibt beim Einschalten immer die selben Daten auf den Sx-Bus
 *
 *
 *  Pin2=T0=INT0, Pin4=T1, Pin3=SX-Write-High, Pin6=SX-Write-Low
 *
 *  Zum Programmieren des Moduls am Sx-Bus können die SX-Kanäle 0 bis 5 verwendet werden:
 *   Sx-Kanal 0 : -
 *   Sx-Kanal 1 : -
 *   Sx-Kanal 2 : -
 *   Sx-Kanal 3 : -
 *   Sx-Kanal 4 : -
 *   Sx-Kanal 5 : -
 *  Die alten Daten auf dem Bus werden gerettet und zurück geschrieben.
 *   
 *  Bit_15 signalisiert die Gleisspannung
 *  Bit_16 ist während der Programmierung an
 *
 *  SXbus.write(adr, val);      to set a value on an address of the SX-bus.
 *  val = SXbus.read(adr);      to get a value from the SXbus.
 *  SXbus.writePWR(val);        val = 0 for off, val = 1 for on (all other values have no influence)
 *  val = SXbus.readPWR();      val = 0 for off, val = 1 for on.
 *  
 */

  // Sx-Teil
  #include "SXArduino.h"                      // Lib einbinden
  SXArduino SXbus(2,4,6,3);                   // Pinbelegung FK (Takt, Daten, Schreiben, Schreiben)
  // SXArduino SXbus(2,4,3,6);                // Pinbelegung openSX(Takt, Daten, Schreiben, Schreiben)
  bool noCC2000 = false;                      // false 0 ... 103, true 0 ... 111

  #include <EEPROM.h>                         // EEPROM Bibliothek hinzufügen

  int analogPin = 6;                          // Progtaster
  #define Bit_1 1                             // Led 1
  #define Bit_2 0                             // Led 2
  #define Bit_3 5                             // Led 3
  #define Bit_4 7                             // Led 4
  #define Bit_5 8                             // Led 5
  #define Bit_6 9                             // Led 6
  #define Bit_7 10                            // Led 7
  #define Bit_8 11                            // Led 8
  #define Bit_15 12                           // Bit_15 Led - Gleis ein
  #define Bit_16 13                           // Bit_16 Led - Prog aktiv

  byte DecAdr=10;                             // Decoder Adresse
  byte SxAdr1=10;                             // Sx Adresse 1
  byte SxData1=0;                             // aktuelle SxDaten 1                  
  byte OldSxData1=6;                          // altes Byte 1
  byte SxAdrSchreib=0;                        // SxAdressen die geschrieben werden 
  byte SxDataSchreib=0;                       // SxDaten die geschrieben werden  
  byte tb;                                    // Gleisbit
  byte oldtb;                                 // alter Wert Gleisbit
  int  progAkt;                               // Programmierung aktiv
  int  Prog = 0;                              // Analogwert Progtaster
  byte Vergleich=0;                           // 
  byte first = 0;                             // 
  byte i;                                     // Zählvariable

void sxisr(void) 
    { 
    SXbus.isr();
    } 

void setup() 
    {
    // Serial.begin(115200);                  // Baudrate
    pinMode(Bit_1,OUTPUT);                    // D0 als Ausgang setzen
    pinMode(Bit_2,OUTPUT);                    // D1 als Ausgang setzen
    pinMode(Bit_3,OUTPUT);                    // D5 als Ausgang setzen
    pinMode(Bit_4,OUTPUT);                    // D7 als Ausgang setzen
    pinMode(Bit_5,OUTPUT);                    // D8 als Ausgang setzen
    pinMode(Bit_6,OUTPUT);                    // D9 als Ausgang setzen
    pinMode(Bit_7,OUTPUT);                    // D10 als Ausgang setzen
    pinMode(Bit_8,OUTPUT);                    // D11 als Ausgang setzen
    pinMode(Bit_15,OUTPUT);                   // D12 als Ausgang setzen
    pinMode(Bit_16,OUTPUT);                   // D13 als Ausgang setzen
    DecAdr = EEPROM.read(1);                  // EEprom Zelle 1 auslesen
    delay(1000);                              // warte
    SXbus.init();                             // SX-bus Kommunikation
    attachInterrupt(0, sxisr, RISING);        // wechsel INT0 triggers the interrupt routine sxisr (see above)
    }

void loop() 
    {
    tb = SXbus.read(109);                     // Info, ob das Gleissignal an/aus ist
    tb = bitRead(tb,7);
    if (oldtb != tb)                          // Gleisbit geändert?
        {
        oldtb = tb;                           // Gleisbit gleich setzen
        digitalWrite(Bit_15,tb);              // Led je nach Zustand ein/ausschalten
        // Serial.println(oldtb);             // Ausgabe
        }

    //if (first == 0 && tb == 1)                  // wird nur 1x beim Ersten Einschalten der Gleisspannung durch laufen
    if (first == 0)                               // wird nur 1x beim Einschalten durch laufen
        {                                         // und schreibt die Werte in ALLE Adressen
        first = 1;                                // wird beim ersten Durchgang gesetzt
        delay(2000);                              // warte 2 sec damit die ZE auch schon bereit ist   
        EEprom_auslesen_Sx_schreiben();           // Unterprogrammaufruf     
        }
       
    // Programmierroutine
     if (progAkt == 0)                            // wenn progAktiv nicht gesetzt
        {
        tb = SXbus.readPWR();                     // Info, ob das Gleissignal an/aus ist
        Prog = analogRead(analogPin);             // read the input pin 
        if (Prog <= 500 && tb ==false)            // falls gedrückt und Gleis aus
            {
            SxData1 = SXbus.read(106);            // Sx-Adr 106 auslesen
            Vergleich = SxData1 & B00100000;      // Prog schon von anderem Decoder gesetzt?
            if (Vergleich != 32)
                {
                progAkt = 1;                      // setze progAktiv 
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                SxDataSchreib = SxData1 | B00100000;  // Prog-Bit setzen
                SxAdrSchreib = 106;               // Sx Adresse die beschrieben werden soll
                Schreiben();                      // ins Unterprog springen
                delay (500);                      // warte
                EEprom_auslesen_Sx_schreiben();   // springe zum Unterprogramm
                }            
            else                                  // Prog bereits gesetzt
                {
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten                
                }
            delay (1500);                         // warte
            // Serial.println(progAkt); 
            }
        } 
    
    if (progAkt == 1)                             // wenn progAktiv gesetzt
        {
        tb = SXbus.readPWR();                     // Info, ob das Gleissignal an/aus ist              
        Prog = analogRead(analogPin);             // read the input pin 
        if (Prog <= 500 || tb ==true)             // falls gedrückt oder Gleis ein
            {
            Sx_auslesen_EEprom_schreiben();       // springe ins Unterprogramm
            SxData1=SXbus.read(106);              // Sx-Adr 106 auslesen
            SxDataSchreib=SxData1 & B11011111;    // Prog-Bit zurück setzen
            SxAdrSchreib=106;                     // Sx Adresse die beschrieben werden soll
            Schreiben();                          // ins Unterprog springen
            delay (1000);                         // warte
            progAkt = 0;                          // lösche progAktiv
            }
        digitalWrite(Bit_16,LOW);                 // Led ausschalten
        }
    }

void Schreiben()
    {    
    SXbus.write(SxAdrSchreib,SxDataSchreib);      // Sx-Bus beschreiben
    } 

void Sx_auslesen_EEprom_schreiben()               // Unterprogramm
    {
    digitalWrite(Bit_1,LOW);                      // Led einschalten
    for (i=0; i <= 103; i++)                      // Schleife von 1 - 103
        {     
        SxData1=SXbus.read(i);                    // Sx-Adr auslesen
        SxDataSchreib=EEPROM.read(i);             // EEprom auslesen
        if (SxData1 != SxDataSchreib)
            {
            EEPROM.write(i,SxData1);              // in eEprom schreiben
            delay(10);                            // warte 10ms bis zum nächsten Senden              
            }
       } 
    digitalWrite(Bit_1,HIGH);                     // Led ausschalten
    }
    
void EEprom_auslesen_Sx_schreiben()               // Unterprogramm
    {
    for (i=0; i < 103; i++)                       // Schleife
        {
        digitalWrite(Bit_1,LOW);                  // Led ausschalten
        delay(80);                                // warte
        SxDataSchreib=EEPROM.read(i);             // EEprom auslesen
        // SxAdrSchreib=i;                        // 
        SXbus.write(i,SxDataSchreib);             // Decoderadr auf Sx-Bus in Adr 0 schreiben                                            
        digitalWrite(Bit_1,HIGH);                 // Led ausschalten
        }
    }
